>{-# OPTIONS -fglasgow-exts #-}

> module CoKleisliArrow(CoKleisliArrow, ReaderArrow) where

> import Arrow
> import ArrowTransformer
> import ArrowOps
> import Comonad

> newtype CoKleisliArrow w a b c = CKA (a (w b) c)
> unCKA (CKA x) = x

> instance (GenFunctor a w, Comonad w) => Arrow (CoKleisliArrow w a) where
>	arr f = CKA (arr (f . eval))
>	CKA f >>> CKA g = CKA (arr cojoin >>> genmap f >>> g)
>	first (CKA f) = CKA (arr lcostrength >>> first f)

> strength :: Functor w => (w a,b) -> w (a,b)
> strength (v, y) = fmap (\x -> (x, y)) v

> lcostrength :: (Functor w, Comonad w) => w (a,b) -> (w a,b)
> lcostrength x = (fmap fst x, snd (eval x))

> rcostrength :: (Functor w, Comonad w) => w (a,b) -> (a,w b)
> rcostrength x = (fst (eval x), fmap snd x)

> instance (GenFunctor a w, Comonad w) =>
>		ArrowTransformer (CoKleisliArrow w) a where
>	lift f = CKA (arr eval >>> f)

> instance ArrowFunctor (CoKleisliArrow w) where
>	arrowmap embed (CKA f) = CKA (embed f)

The following promotions follow directly from the arrow transformer.

> instance (ArrowZero a, GenFunctor a w, Comonad w) =>
>		ArrowZero (CoKleisliArrow w a) where
>	zeroArrow = lift zeroArrow

> instance (ArrowCircuit a, GenFunctor a w, Comonad w) =>
>		ArrowCircuit (CoKleisliArrow w a) where
>	delay x = lift (delay x)

> instance (ArrowExcept ex a, GenFunctor a w, Comonad w) =>
>		ArrowExcept ex (CoKleisliArrow w a) where
>       raise = lift raise

> instance (ArrowState s a, GenFunctor a w, Comonad w) =>
>		ArrowState s (CoKleisliArrow w a) where
>       fetch = lift fetch
>       store = lift store

> instance (ArrowWriter s a, GenFunctor a w, Comonad w) =>
>		ArrowWriter s (CoKleisliArrow w a) where
>       write = lift write

liftings of standard classes

> instance (GenFunctor a w, ArrowLoop a, Comonad w) =>
>		ArrowLoop (CoKleisliArrow w a) where
>	loop (CKA f) = CKA (loop (arr strength >>> f))

> instance (GenFunctor a w, ArrowPlus a, Comonad w) =>
>		ArrowPlus (CoKleisliArrow w a) where
>	CKA f <+> CKA g = CKA (f <+> g)

All operators promote to the new arrow.

> instance (GenFunctor a w, GenFunctor a' w, ArrowRunState s a a', Comonad w) =>
>		ArrowRunState s (CoKleisliArrow w a) (CoKleisliArrow w a') where
>	runState (CKA f) = CKA (arr lcostrength >>> runState f)

instance (GenFunctor a w, GenFunctor a' w, ArrowRunReader r a a', Comonad w) =>
		ArrowRunReader r (CoKleisliArrow w a) (CoKleisliArrow w a') where
	runReader (CKA f) = CKA (arr lcostrength >>> runReader f)

> instance (GenFunctor a w, GenFunctor a' w, ArrowRunWriter s a a', Comonad w) =>
>		ArrowRunWriter s (CoKleisliArrow w a) (CoKleisliArrow w a') where
>	runWriter (CKA f) = CKA (runWriter f)

> instance (GenFunctor a w, ArrowNewWriter s a, Comonad w) =>
>		ArrowNewWriter s (CoKleisliArrow w a) where
>	newWriter (CKA f) = CKA (newWriter f)

> instance (GenFunctor a w, GenFunctor a' w, ArrowRunExcept ex a a', Comonad w) =>
>		ArrowRunExcept ex (CoKleisliArrow w a) (CoKleisliArrow w a') where
>	runExcept (CKA f) (CKA h) = CKA (runExcept f (arr strength >>> h))

> instance (GenFunctor a w, ArrowNewExcept ex a, Comonad w) =>
>		ArrowNewExcept ex (CoKleisliArrow w a) where
>	newExcept (CKA f) (CKA h) = CKA (newExcept f (arr strength >>> h))

> instance (GenFunctor a w, GenFunctor a' w, ArrowRunMaybe a a', Comonad w) =>
>		ArrowRunMaybe (CoKleisliArrow w a) (CoKleisliArrow w a') where
>	runMaybe (CKA f) (CKA g) = CKA (runMaybe f g)

Readers

> type ReaderArrow r = CoKleisliArrow ((,) r)

> instance ArrowChoice a => ArrowChoice (ReaderArrow r a) where
>	left (CKA f) = CKA (arr dist' >>> left f)
>		where	dist' :: (s, Either b c) -> Either (s, b) c
>			dist' (s, Left x) = Left (s, x)
>			dist' (s, Right y) = Right y

> instance ArrowApply a => ArrowApply (ReaderArrow r a) where
>	app = CKA (arr (\(r, (CKA f, x)) -> (f, (r, x))) >>> app)

> instance Arrow a => ArrowReader r (ReaderArrow r a) where
>	readState = CKA (arr fst)

> instance Arrow a => ArrowRunReader r (ReaderArrow r a) a where
>	runReader (CKA f) = arr swap >>> f
>		where	swap ~(x,y) = (y,x)

Replace the current state

> instance Arrow a => ArrowNewReader r (ReaderArrow r a) where
>	newReader (CKA f) = CKA (arr (\(r, (x, _)) -> (r, x)) >>> f)
